# -------------------------------
# PipeBit
# '__init__.py'
# Author: Juan Carlos Juárez.
# Licensed under MPL 2.0.
# All rights reserved.
# -------------------------------

from .bitpacksender import BitPackSender
from .bitpackreceiver import BitPackReceiver


# -------------------------------
# PipeBit
# 'bitpacksender.py'
# Author: Juan Carlos Juárez.
# Licensed under MPL 2.0.
# All rights reserved.
# -------------------------------

from xmlrpc.client import Boolean
import pipebit.initial_config as initial_config
import sys
import random
import string

class BitPackSender:

    pipeline_packet_size = 0
    pipeline_name = ""
    pipeline_path = ""
    pipeline_connection = False
    pipeline_data_delimeter = "_%_"
    pipeline_packet_delimeter = "/*/"

    def __init__(self, _name, _packet_size, _override):
        if(_name == ""):
            print("\n<#> PipeBit Error: Pipeline Name cannot be empty.")
            sys.exit(1)
        if(_packet_size < 1):
            print("\n<#> PipeBit Error: Pipeline Packet Size must be at least 1.")
            sys.exit(1)
        if(type(_override) != Boolean):
            print("\n<#> PipeBit Error: Pipeline Override Parameter must be Boolean.")
            sys.exit(1)
        self.pipeline_packet_size = _packet_size
        self.pipeline_name = str(_name)
        self.pipeline_path = str(initial_config.set(_name, _packet_size, _override, self.pipeline_data_delimeter, self.pipeline_packet_delimeter))
        print("\n<+> PipeBit Info: The BitPack Sender Pipeline '{0}' has been created succesfully.".format(str(self.pipeline_name)))

    def open_connection(self):
        self.pipeline_connection = True

    def close_connection(self):
        self.pipeline_connection = False

    def send(self, data):
        if(len(data) != self.pipeline_packet_size): 
            print("\n<#> PipeBit Error: Packet Size is different from number of arguments in the passed data.")
            return
        if(not self.pipeline_connection):
            print("\n<#> PipeBit Error: Pipeline Connection is closed.")
            return

        # Generate 16-character Transaction    
        transaction = ''.join(random.choice(string.ascii_lowercase + string.digits) for _ in range(16))
        binary_transaction = bytes(transaction + "\n", 'utf-8')
        with open(self.pipeline_path, "wb") as binary_file:
            binary_file.write(binary_transaction)
        for arg in data:
            arg = str(arg) + self.pipeline_data_delimeter
            binary_data = bytes(arg, 'utf-8')
            with open(self.pipeline_path, "ab") as binary_file:
                binary_file.write(binary_data)
        binary_packet_delimiter = bytes(self.pipeline_packet_delimeter, 'utf-8')
        with open(self.pipeline_path, "ab") as binary_file:
                binary_file.write(binary_packet_delimiter)


# -------------------------------
# PipeBit
# 'initial_config.py'
# Author: Juan Carlos Juárez.
# Licensed under MPL 2.0.
# All rights reserved.
# -------------------------------

import os
import sys
import inspect

def set(pipeline_name, packet_size, override, data_delimiter, packet_delimiter):
    args = [pipeline_name, packet_size, data_delimiter, packet_delimiter]
    pipebit_dir = str(os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe()))))
    pipebit_path = repr(str(pipebit_dir))[1:-1]
    pipebit_pipelines_dir = "pipelines"
    pipebit_pipelines_path = os.path.join(pipebit_path, pipebit_pipelines_dir)
    if(not(os.path.isdir(pipebit_pipelines_path))):
        try:
            os.mkdir(pipebit_pipelines_path)
        except:
            print("\n<#> PipeBit Error: PipeBit Pipelines Directory cannot be found or created.")
            sys.exit(1)
    pipebit_current_pipeline_dir_path = os.path.join(pipebit_pipelines_path, pipeline_name)
    pipebit_current_pipeline_file_path = ""
    pipebit_current_pipeline_info_path = ""

    # Pipeline does not exist

    if(not(os.path.isdir(pipebit_current_pipeline_dir_path))):
        try:
            os.mkdir(pipebit_current_pipeline_dir_path)
        except:
            print("\n<#> PipeBit Error: Current Pipeline Directory cannot be created.")
            sys.exit(1)
        pipebit_current_pipeline_file_path = os.path.join(pipebit_current_pipeline_dir_path, pipeline_name)
        pipebit_current_pipeline_info_path = os.path.join(pipebit_current_pipeline_dir_path, "info.txt")
        try:
            with open(pipebit_current_pipeline_file_path, "wb") as pipeline_file:
                pass
        except:
            print("\n<#> PipeBit Error: Current Pipeline cannot be created.")
            sys.exit(1)
        try:
            with open(pipebit_current_pipeline_info_path, "w") as info_file:
                for arg in args:
                    info_file.write("{0}\n".format(str(arg)))
                info_file.write(pipebit_current_pipeline_file_path)
        except:
            print("\n<#> PipeBit Error: Current Pipeline Information File cannot be created.")
            sys.exit(1)
        return pipebit_current_pipeline_file_path

    # Pipeline Already exists

    pipebit_current_pipeline_file_path = os.path.join(pipebit_current_pipeline_dir_path, pipeline_name)
    pipebit_current_pipeline_info_path = os.path.join(pipebit_current_pipeline_dir_path, "info.txt")

    # Override Pipeline

    if(override):
        try:
            with open(pipebit_current_pipeline_file_path, "wb") as pipeline_file:
               print("\n<!> PipeBit Alert: [OVERRIDE] This Pipeline already exists. This Pipeline has been overriden. You can navigate to: '{0}' to delete the pipelines you no longer need.".format(str(pipebit_pipelines_path))) 
        except:
            print("\n<#> PipeBit Error: Current Pipeline cannot be created.")
            sys.exit(1)
        try:
            with open(pipebit_current_pipeline_info_path, "w") as info_file:
                for arg in args:
                    info_file.write("{0}\n".format(str(arg)))
                info_file.write(pipebit_current_pipeline_file_path)
        except:
            print("\n<#> PipeBit Error: Current Pipeline Information File cannot be created.")
            sys.exit(1)
        return pipebit_current_pipeline_file_path

    # Do not Override Pipeline

    try:
        with open(pipebit_current_pipeline_file_path, "ab") as pipeline_file:
            print("\n<!> PipeBit Alert: [NO-OVERRIDE] This Pipeline already exists. A direct reference to this pipeline has been generated. You can navigate to: '{0}' to delete the pipelines you no longer need.".format(str(pipebit_pipelines_path)))
    except:
        print("\n<#> PipeBit Error: Current Pipeline cannot be created.")
        sys.exit(1)
    try:
        with open(pipebit_current_pipeline_info_path, "w") as info_file:
            for arg in args:
                info_file.write("{0}\n".format(str(arg)))
            info_file.write(str(pipebit_current_pipeline_file_path))
    except:
        print("\n<#> PipeBit Error: Current Pipeline Information File cannot be created.")
        sys.exit(1)
    return pipebit_current_pipeline_file_path


# -------------------------------
# PipeBit
# 'bitpackreceiver.py'
# Author: Juan Carlos Juárez.
# Licensed under MPL 2.0.
# All rights reserved.
# -------------------------------

import os
import sys
import inspect

class BitPackReceiver:

    pipeline_packet_size = 0
    pipeline_name = ""
    pipeline_path = ""
    pipeline_connection = False
    pipeline_data_delimeter = "_%_"
    pipeline_packet_delimeter = "/*/"

    def __init__(self, _name):
        if(_name == ""):
            print("\n<#> PipeBit Error: Pipeline Name cannot be empty.")
            sys.exit(1)
        pipebit_dir = str(os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe()))))
        pipebit_path = repr(str(pipebit_dir))[1:-1]
        pipebit_pipelines_dir = "pipelines"
        pipebit_pipelines_path = os.path.join(pipebit_path, pipebit_pipelines_dir)
        pipebit_current_pipeline_dir_path = os.path.join(pipebit_pipelines_path, _name)
        if(not(os.path.isdir(pipebit_current_pipeline_dir_path))):
            print("\n<#> PipeBit Error: Pipeline '{0}' does not exist.".format(_name))
            sys.exit(1)
        pipebit_current_pipeline_info_path = os.path.join(pipebit_current_pipeline_dir_path, "info.txt")
        if(not(os.path.isfile(pipebit_current_pipeline_info_path))):
            print("\n<#> PipeBit Error: Missing Information File for Pipeline '{0}'. Try to recreate the Pipeline Sender again.".format(_name))
            sys.exit(1)
        params = []
        with open(pipebit_current_pipeline_info_path, "r") as info_file:
            for line in info_file:
                params.append(line.rstrip('\n'))
        self.pipeline_name = str(params[0])
        self.pipeline_packet_size = int(params[1])
        self.pipeline_data_delimeter = str(params[2])
        self.pipeline_packet_delimeter = str(params[3])
        self.pipeline_path = str(params[4])
        print("\n<+> PipeBit Info: The BitPack Receiver Pipeline '{0}' has been created succesfully.".format(str(self.pipeline_name)))

    def open_connection(self):
        self.pipeline_connection = True

    def close_connection(self):
        self.pipeline_connection = False

    def receive(self):
        res = []
        curr = ""
        is_size = False
        while(not is_size):
            lines = []
            with open(self.pipeline_path, "rb") as binary_file:
                for line in binary_file:
                    curr = line.decode("utf-8").rstrip('\n')
                    lines.append(curr)
            transaction = lines[0]
            res = curr[1]
            res = curr.split("_%_")
            res.pop()
            if(len(res) == self.pipeline_packet_size): is_size = True
        return res          